@@ -7,6 +7,7 @@ import importlib
 import pytest
 
 EPSILON = 0.0000000001
+EPSILON2 = 0.000000001
 
 
 def cross_py23_reload(module):
@@ -59,8 +60,8 @@ def test_fail_bad_tform():
         tform = renderapi.transform.load_transform_json(d)  # noqa: F841
 
 
-def test_affine_rot_90():
-    am = renderapi.transform.AffineModel()
+def test_similarity_rot_90():
+    am = renderapi.transform.SimilarityModel()
     # setup a 90 degree clockwise rotation
     points_in = np.array([[0, 0], [0, 1], [1, 0], [1, 1]], np.float)
     points_out = np.array([[0, 0], [1, 0], [0, -1], [1, -1]], np.float)
@@ -71,7 +72,7 @@ def test_affine_rot_90():
     assert(np.abs(am.rotation + np.pi / 2) < .000001)
     assert(np.abs(am.translation[0]) < .000001)
     assert(np.abs(am.translation[1]) < .000001)
-    assert(np.abs(am.shear) < .000001)
+    assert(np.all(np.abs(am.shear) < .000001))
 
     points = np.array([[20, 30], [1, 2], [10, -5], [-4, 3], [5.6, 2.3]])
     new_points = am.tform(points)
@@ -88,7 +89,7 @@ def test_affine_rot_90():
     assert(np.abs(identity.rotation) < .000001)
     assert(np.abs(identity.translation[0]) < .000001)
     assert(np.abs(identity.translation[1]) < .000001)
-    assert(np.abs(identity.shear) < .000001)
+    assert(np.all(np.abs(identity.shear) < .000001))
     print(str(am))
 
 
@@ -102,13 +103,41 @@ def test_affine_fail():
         am.estimate(points_in, points_out[0:-2, :])
 
 
+def test_json_affine():
+    M00 = .9
+    M10 = -0.2
+    M01 = 0.3
+    M11 = .85
+    B0 = 245.3
+    B1 = -234.1
+    am = renderapi.transform.AffineModel(M00=M00,
+                                         M10=M10,
+                                         M01=M01,
+                                         M11=M11,
+                                         B0=B0,
+                                         B1=B1)
+
+    props = np.array(am.calc_properties())
+    jam = am.to_dict()
+    am2 = renderapi.transform.AffineModel(json=jam)
+    jprops = np.array(am2.calc_properties())
+
+    assert(np.all(np.abs(props - jprops) < EPSILON))
+
+
 def test_affine_random():
-    am = renderapi.transform.AffineModel(M00=.9,
-                                         M10=-0.2,
-                                         M01=0.3,
-                                         M11=.85,
-                                         B0=245.3,
-                                         B1=-234.1)
+    M00 = .9
+    M10 = -0.2
+    M01 = 0.3
+    M11 = .85
+    B0 = 245.3
+    B1 = -234.1
+    am = renderapi.transform.AffineModel(M00=M00,
+                                         M10=M10,
+                                         M01=M01,
+                                         M11=M11,
+                                         B0=B0,
+                                         B1=B1)
 
     points_in = np.random.rand(10, 2)
     points_out = am.tform(points_in)
@@ -118,6 +147,13 @@ def test_affine_random():
 
     assert(np.sum(np.abs(am.M.ravel() - am_fit.M.ravel())) < (.001 * 6))
 
+    # return_all test
+    tvec, res, rank, s = am.fit(points_in, points_out, return_all=True)
+    compare_tvec = np.array([M00, M01, M10, M11, B0, B1])
+    tvec = tvec.flatten()
+    assert(np.linalg.norm(tvec - compare_tvec) < EPSILON)
+    assert(res < EPSILON)
+
 
 def test_invert_Affine():
     am = renderapi.transform.AffineModel(M00=.9,
@@ -627,6 +663,133 @@ def test_similarity_init():
     assert(tform.M[0, 2] == 10)
 
 
+def test_affine_properties():
+    # scale
+    sx = 1.5
+    sy = 1.1
+    # shear
+    cx = 0.0
+    cy = 0.0
+    # angle
+    theta = 0.0
+    pt = np.array(
+        [1.234, 4.567, 1])
+    M, testpt, rnd_tf = affine_property_function(sx, sy, cx, cy, theta, pt)
+    tformpt = rnd_tf.tform(np.array([pt[0:2]]))
+    assert np.all(np.abs(M - rnd_tf.M) < 1e-8)
+    assert np.all(np.abs(testpt[0:2] - tformpt) < 1e-8)
+    assert (np.abs(rnd_tf.scale[0] - sx) < 1e-8) & \
+           (np.abs(rnd_tf.scale[1] - sy) < 1e-8)
+    assert np.abs(rnd_tf.rotation - theta) < 1e-8
+    assert (np.abs(rnd_tf.shear - cx) < 1e-8)
+    rnd_tf.force_shear = 'y'
+    assert (np.abs(rnd_tf.scale[0] - sx) < 1e-8) & \
+           (np.abs(rnd_tf.scale[1] - sy) < 1e-8)
+    assert np.abs(rnd_tf.rotation - theta) < 1e-8
+    assert (np.abs(rnd_tf.shear - cx) < 1e-8)
+
+    theta = 0.1234
+    pt = np.array(
+        [1.234, 4.567, 1])
+    M, testpt, rnd_tf = affine_property_function(sx, sy, cx, cy, theta, pt)
+    tformpt = rnd_tf.tform(np.array([pt[0:2]]))
+    assert np.all(np.abs(M - rnd_tf.M) < 1e-8)
+    assert np.all(np.abs(testpt[0:2] - tformpt) < 1e-8)
+    assert (np.abs(rnd_tf.scale[0] - sx) < 1e-8) & \
+           (np.abs(rnd_tf.scale[1] - sy) < 1e-8)
+    assert np.abs(rnd_tf.rotation - theta) < 1e-8
+    assert (np.abs(rnd_tf.shear - cx) < 1e-8)
+    rnd_tf.force_shear = 'y'
+    assert (np.abs(rnd_tf.scale[0] - sx) < 1e-8) & \
+           (np.abs(rnd_tf.scale[1] - sy) < 1e-8)
+    assert np.abs(rnd_tf.rotation - theta) < 1e-8
+    assert (np.abs(rnd_tf.shear - cx) < 1e-8)
+
+    cx = 0.2
+    cy = 0.0
+    M, testpt, rnd_tf = affine_property_function(sx, sy, cx, cy, theta, pt)
+    tformpt = rnd_tf.tform(np.array([pt[0:2]]))
+    assert np.all(np.abs(M - rnd_tf.M) < 1e-8)
+    assert np.all(np.abs(testpt[0:2] - tformpt) < 1e-8)
+    assert (np.abs(rnd_tf.scale[0] - sx) < 1e-8) & \
+           (np.abs(rnd_tf.scale[1] - sy) < 1e-8)
+    assert np.abs(rnd_tf.rotation - theta) < 1e-8
+    assert (np.abs(rnd_tf.shear - cx) < 1e-8)
+
+    rnd_tf.force_shear = 'y'
+    with pytest.raises(AssertionError):
+        assert (np.abs(rnd_tf.scale[0] - sx) < 1e-8) & \
+               (np.abs(rnd_tf.scale[1] - sy) < 1e-8)
+    with pytest.raises(AssertionError):
+        assert np.abs(rnd_tf.rotation - theta) < 1e-8
+
+    cx = 0.0
+    cy = 0.1
+    M, testpt, rnd_tf = affine_property_function(sx, sy, cx, cy, theta, pt)
+    tformpt = rnd_tf.tform(np.array([pt[0:2]]))
+    assert np.all(np.abs(M - rnd_tf.M) < 1e-8)
+    assert np.all(np.abs(testpt[0:2] - tformpt) < 1e-8)
+    with pytest.raises(AssertionError):
+        assert (np.abs(rnd_tf.scale[0] - sx) < 1e-8) & \
+               (np.abs(rnd_tf.scale[1] - sy) < 1e-8)
+    with pytest.raises(AssertionError):
+        assert np.abs(rnd_tf.rotation - theta) < 1e-8
+
+    rnd_tf.force_shear = 'y'
+    assert (np.abs(rnd_tf.scale[0] - sx) < 1e-8) & \
+           (np.abs(rnd_tf.scale[1] - sy) < 1e-8)
+    assert np.abs(rnd_tf.rotation - theta) < 1e-8
+    assert (np.abs(rnd_tf.shear - cy) < 1e-8)
+
+    cx = 0.3
+    cy = 0.1
+    M, testpt, rnd_tf = affine_property_function(sx, sy, cx, cy, theta, pt)
+    tformpt = rnd_tf.tform(np.array([pt[0:2]]))
+    assert np.all(np.abs(M - rnd_tf.M) < 1e-8)
+    assert np.all(np.abs(testpt[0:2] - tformpt) < 1e-8)
+    with pytest.raises(AssertionError):
+        assert (np.abs(rnd_tf.scale[0] - sx) < 1e-8) & \
+               (np.abs(rnd_tf.scale[1] - sy) < 1e-8)
+    with pytest.raises(AssertionError):
+        assert np.abs(rnd_tf.rotation - theta) < 1e-8
+    with pytest.raises(AssertionError):
+        assert (np.abs(rnd_tf.shear - cx) < 1e-8)
+
+    rnd_tf.force_shear = 'y'
+    with pytest.raises(AssertionError):
+        assert (np.abs(rnd_tf.scale[0] - sx) < 1e-8) & \
+               (np.abs(rnd_tf.scale[1] - sy) < 1e-8)
+    with pytest.raises(AssertionError):
+        assert np.abs(rnd_tf.rotation - theta) < 1e-8
+    with pytest.raises(AssertionError):
+        assert (np.abs(rnd_tf.shear - cy) < 1e-8)
+
+
+def affine_property_function(sx, sy, cx, cy, theta, pt):
+    scale = np.array([
+        [sx, 0, 0],
+        [0, sy, 0],
+        [0, 0, 1]])
+    shear = np.array([
+        [1, cx, 0],
+        [cy, 1, 0],
+        [0, 0, 1]])
+    rotation = np.array([
+        [np.cos(theta), -np.sin(theta), 0],
+        [np.sin(theta), np.cos(theta), 0],
+        [0, 0, 1]])
+    M = scale.dot(shear.dot(rotation))
+
+    testpt = M.dot(pt)[0:2]
+
+    rnd_tf = renderapi.transform.SimilarityModel(
+            M00=M[0, 0],
+            M01=M[0, 1],
+            M10=M[1, 0],
+            M11=M[1, 1])
+    return M, testpt, rnd_tf
+
+
 def test_thinplatespline_inverse():
     j = json.load(open(rendersettings.TEST_THINPLATESPLINE_FILE, 'r'))
     t = renderapi.transform.ThinPlateSplineTransform(
@@ -677,6 +840,104 @@ def test_thinplatespline():
     dst_pts = t.tform(src_pts)
     assert(dst_pts.shape == src_pts.shape)
 
+    # check load from json
+    jt = t.to_dict()
+    nt = renderapi.transform.ThinPlateSplineTransform(
+            json=jt)
+    assert nt == t
+
+
+def test_thinplatespline_apply():
+    # tests some copied behavior from trakem2
+    j = json.load(open(rendersettings.TEST_THINPLATESPLINE_FILE, 'r'))
+    t = renderapi.transform.ThinPlateSplineTransform(
+            dataString=j['dataString'])
+    del t.dMtxDat
+    pt = np.array([1.234, 45.678])
+    npt = t.apply(pt)
+    assert np.all(pt == npt)
+
+
+def estimate_test(jpath, computeAffine=True):
+    # test that the estimate method can produce same results
+    with open(jpath, 'r') as f:
+        j = json.load(f)
+
+    t = renderapi.transform.ThinPlateSplineTransform(
+            dataString=j['dataString'])
+    # exact points
+    src1 = np.transpose(t.srcPts)
+    # some in-between points
+    x = np.linspace(
+            src1[:, 0].min(),
+            src1[:, 0].max(),
+            int(np.sqrt(t.nLm)) * 3)
+    y = np.linspace(
+            src1[:, 1].min(),
+            src1[:, 1].max(),
+            int(np.sqrt(t.nLm)) * 3)
+    xt, yt = np.meshgrid(x, y)
+    src2 = np.transpose(np.vstack((xt.flatten(), yt.flatten())))
+    dst1_a = t.tform(src1)
+    dst2_a = t.tform(src2)
+
+    # estimate
+    t.estimate(src1, dst1_a, computeAffine=computeAffine)
+    dst1_b = t.tform(src1)
+    dst2_b = t.tform(src2)
+    delta1 = np.linalg.norm(dst1_b - dst1_a, axis=1)
+    delta2 = np.linalg.norm(dst2_b - dst2_a, axis=1)
+
+    assert delta1.max() < EPSILON2
+    assert delta2.max() < EPSILON2
+
+    with pytest.raises(renderapi.errors.EstimationError):
+        t.estimate(src1, dst1_a[1:, :])
+
+
+def test_thinplatespline_estimate():
+    estimate_test(
+            rendersettings.TEST_THINPLATESPLINE_FILE,
+            computeAffine=False)
+    estimate_test(
+            rendersettings.TEST_THINPLATESPLINEAFFINE_FILE,
+            computeAffine=True)
+    thinplate_estimate_nojson(computeAffine=True)
+    thinplate_estimate_nojson(computeAffine=False)
+
+
+def thinplate_estimate_nojson(computeAffine=True):
+    # an estimate test that does not depend on pre-computed json
+    x = np.linspace(0, 1000, 40)
+    xt, yt = np.meshgrid(x, x)
+    src = np.transpose(np.vstack((xt.flatten(), yt.flatten())))
+
+    def poly2d(src):
+        dst = np.zeros_like(src)
+        dx = (src[:, 0] - src[:, 0].mean()) / (src[:, 0].ptp())
+        dy = (src[:, 1] - src[:, 1].mean()) / (src[:, 1].ptp())
+        dst[:, 0] = src[:, 0] + 0.5 * dx * dy + 7.0 * dx * dy * dy
+        dst[:, 1] = src[:, 1] + 0.7 * dy * dy - 4.0 * dx * dx * dy
+        a = np.array([[1.1, -0.04], [-0.02, 0.95]])
+        b = np.array([3.0, 4.0])
+        dst = np.transpose(a.dot(np.transpose(dst))) + b
+        return dst
+
+    dst = poly2d(src)
+    t = renderapi.transform.ThinPlateSplineTransform()
+    t.estimate(src, dst, computeAffine=computeAffine)
+
+    x = np.linspace(0, 1000, 120)
+    xt, yt = np.meshgrid(x, x)
+    test_src = np.transpose(np.vstack((xt.flatten(), yt.flatten())))
+    p_dst = poly2d(test_src)
+    t_dst = t.tform(test_src)
+
+    delta = np.linalg.norm(p_dst - t_dst, axis=1)
+    # can it match the polynomial within a pixel?
+    # for low-N, it won't
+    assert delta.max() < 1.0
+
 
 def test_encode64():
     # case for Stephan's '@' character
